import torch
import torch.nn as nn
from torch.nn.modules import conv


class conv_block(nn.Module):
    def __init__(self, in_channels, out_channels, **kwargs):
        super(conv_block, self).__init__()

        self.conv_layer = nn.Sequential(
            nn.Conv2d(in_channels, out_channels, **kwargs),
            nn.BatchNorm2d(out_channels),
            nn.ReLU(),
        )
    def forward(self, x):
        return self.conv_layer(x)


class KimCNN(nn.Module):
    def __init__(self, aux_logits=True, num_classes=2, init_weights=True):
        super(GoogLeNet, self).__init__()
        assert aux_logits == True or aux_logits == False
        self.aux_logits = aux_logits

        # input=3x224x224
        self.conv1 = conv_block(3, 64, kernel_size=3, stride=1, padding=2)
        self.maxpool1 = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        self.conv2 = conv_block(64, 192, kernel_size=3, stride=1, padding=1)
        self.maxpool2 = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        self.maxpool3 = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        self.conv3 = conv_block(64, 192, kernel_size=3, stride=1, padding=1)
        self.conv4 = conv_block(64, 192, kernel_size=3, stride=1, padding=1)
     

        self.maxpool4 = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)

        self.avgpool = nn.AvgPool2d(kernel_size=7, stride=1)
        self.dropout = nn.Dropout(p=0.4)
        self.fc1 = nn.Linear(1024)
        self.fc1 = nn.Linear()
        self.fc1 = nn.Linear(1024)

        if init_weights:
            self._initialize_weights()
        
    def forward(self, x):
        x = self.conv1(x)
        x = self.maxpool1(x)
        x = self.conv2(x)
        x = self.maxpool2(x)
        x = self.maxpool3(x)
        if self.aux_logits and self.training:
            aux1 = self.aux1(x)


        if self.aux_logits and self.training:
            aux2 = self.aux2(x)

        x = self.maxpool4(x)
        x = self.avgpool(x)

        x = x.view(x.shape[0], -1)

        x = self.dropout(x)
        x = self.fc1(x)

        if self.aux_logits and self.training:
            return x, aux1, aux2
        else:
            return x

    # define weight initialization function
    def _initialize_weights(self):
        for m in self.modules():
            if isinstance(m, nn.Conv2d):
                nn.init.kaiming_normal_(m.weight, mode='fan_out', nonlinearity='relu')
                if m.bias is not None:
                    nn.init.constant_(m.bias, 0)
            elif isinstance(m, nn.BatchNorm2d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 0)
            elif isinstance(m, nn.Linear):
                nn.init.normal_(m.weight, 0, 0.01)
                nn.init.constant_(m.bias, 0)


# # for testing whether the network is working
# model = GoogLeNet().to("cuda")
# x = torch.rand((3, 3, 224, 224)).to("cuda")
# print("input:", x.shape)
# output = model(x)
# print(output)

# ## softmax practice
# print(type(output))  # tuple
# print(type(output[0]))  # tensor
# softmax = nn.Softmax(dim=1)
# p = softmax(output[0])
# print(p)
# sum = torch.sum(p, dim=1)
# print(sum)